//-------------------------------------------------------------//
//              Autumn Hill Software Incorporated              //
//          Copyright (c) 1991 -- All Rights Reserved          //
//-------------------------------------------------------------//
// File:   Demo.Cpp                                            //
// Desc:   Menuet/CPP V1.1 Demo Program                        //
// Date:   Feb, 1992                                           //
//-------------------------------------------------------------//

#include "mstdhdr.h"

#include "ahslogo.hpp"

extern unsigned _stklen = 12000;

//--------------------------------------------------------------------//

// some special colors

static int gold = 5;
static int blue = 8;

// auxiliary fonts used by the demo

mFont *flge = 0,
      *fsys = 0,
      *fsml = 0;

// buffers for file dialog

char filepath[81] = "*.BMP";
char filepick[81] = "";

//--------------------------------------------------------------------//

// main menu QUIT processing

char *quit_text[] =
{
   " Quit the Menuet/CPP Demo? "
};

mTextPg quit_pg( "QUIT", quit_text, 1 );

// callback for the query dialog
void quit_respond( mWindowManager& wm, int resp )
{
   if( resp == OKAY )
      wm << mMsg( mMANAGER, mCLOSE );
}

void do_quit( mWnControl& ctl )
{
   mWindowManager *wm = ctl.getwind()->getwnmgr();
   mWindow *qwn = create_query_window( &wm->systemfont(),
                                       &quit_pg, quit_respond );
   // figure out a good place to open the dialog
   mRect r1 = ctl.getwind()->extent() + ctl.getwind()->getwnorigin();
   mRect r2 = qwn->extent();
   mPoint p( r1.midx()-r2.midx(), r1.midy()-r2.midy() );
   qwn->setwnorigin( p );
   // and open it
   (*wm) << (*qwn);
}

//--------------------------------------------------------------------//

// a generic okay-button task for closing windows

int button_task( mMsg& msg, mWnControl& ctl )
{
   if( (msg.verb == mEXECUTE) || (msg.verb == mENDEXECUTE) )
   {
      (*ctl.getwind()->getwnmgr()) << mMsg(mWINDOW,mCLOSE);
      return 1;
   }
   return 0;
}

//--------------------------------------------------------------------//

// a generic enter/escape window task

int window_task( mMsg& msg, mWindow& win )
{
   if( msg.noun != mEVT ) return 0;
   mEvent *evt = (mEvent *) msg.msgpointer();
   if( (evt->devtype == devKEYBRD) &&
       ((evt->key.keyvalue == kbENTER) || (evt->key.keyvalue == kbESCAPE)) )
      (*win.getwnmgr()) << mMsg(mWINDOW,mCLOSE);
   return 0;
}

//--------------------------------------------------------------------//

// task for info wind - draws the logo icon

int info_image_task( mMsg& msg, mWnControl& ctl )
{
   if( msg.verb == mENDDRAW )
   {
      (*ctl.getwind()->getwnmgr()) << mMsg(mMOUSE,mHIDECURSOR);
      draw_logo( mPoint(ctl.extent().xmin+2, ctl.extent().ymin+2) );
      (*ctl.getwind()->getwnmgr()) << mMsg(mMOUSE,mSHOWCURSOR);
   }
   return 0;
}

//--------------------------------------------------------------------//

// main menu INFO processing

static char *info_titl =
   "Menuet/CPP";

static char *info_text[] =
{
   "A Multi-Platform C++",
   "GUI Application Framework",
   "Autumn Hill Software Incorporated",
   "1145 Ithaca Drive",
   "Boulder, Colorado 80303 USA",
   "Tel 303-494-8865",
   "Fax 303-494-7802",
};

static mPoint *info_origin = 0;

#define LHGT 320

void do_info( mWindowManager *wm )
{
    // determine a good size
    mPoint p = logo_size();
    int    h = p.y;
    mPoint q = GetWnSize( p.x+300, p.y+20, wcTITLE );

    // create the info window
    mWindow *wn = new mWindow( "ABOUT", 0, &wm->systemfont(),
                               q.x, q.y, wBDRFIXED );
    wn->setstatus( yDESTROY, 1 );
    wn->setclienttask( window_task );
    mRect r = wn->getclientrect();
    if( info_origin == 0 )
    {
       info_origin = new mPoint( 0, 0 );
       mRect c = wm->curwindow().getclientrect();
       info_origin->x = c.midx() - (wn->extent().delx() / 2 );
       info_origin->y = c.midy() - (wn->extent().dely() / 2 );
    }
    wn->setwnorigin( *info_origin );
    mRect s( r.xmin+10, r.ymin+10,  r.xmin+p.x+9, r.ymin+p.y+9 );
    s <<= 2;
    mWnCtlAperture *ap = new mWnCtlAperture( s, RGNBOX );
    ap->settask( info_image_task );
    ap->setstatus( xPOSTDRAW, 1 );
    (*wn) << (*ap);

    // some descriptive text
    s.set( s.xmax+10, r.ymin+60, r.xmax-10, s.ymax );
    p.set( s.midx() - flge->stringwidth(info_titl)/2,
           s.ymax - iscale(50,h,LHGT) );
    mWnCtlText *ct = new mWnCtlText( p, flge, info_titl );
    ct->getnodes()->txt->setpen( mPen(0,15,rgnOUTSET,1) );
    *wn << *ct;

    p.y -= iscale( 40, h, LHGT);
    int nlines = sizeof( info_text ) / sizeof( char * );
    for( int i=0; i<nlines; i++ )
    {
       p.x = s.midx() - fsml->stringwidth( info_text[i] )/2;
       ct = new mWnCtlText( p, fsml, info_text[i] );
       *wn << *ct;
       p.y -= iscale( 28, h, LHGT);
    }

    s.set( s.midx()-40, s.ymin-37, s.midx()+40, s.ymin-15 );
    mWnCtlButton *but = new mWnCtlButton( s, &wm->systemfont(), "Okay" );
    but->settask( button_task );
    *wn << *but;

    *wm << *wn;
}

//--------------------------------------------------------------------//

// function to create a text page window

void do_textpage( mWindowManager& mgr, mFont& fnt,
                  char *title, char *txt[], int nlines )
{
    // determine a good window size
    int h = (fnt.getcellh() + 1) * nlines + 100;
    int w = 0;
    for( int i=0; i<nlines; i++ )
    {
       int n = fnt.stringwidth( txt[i] );
       if( n > w ) w = n;
    }
    w += 40;

    // create the window
    mWindow *wn = new
          mWindow( title, 0, mgr.getsysfont(), w, h, wBDRSIZABLE );
    wn->setstatus( yDESTROY, 1 );
    wn->setclienttask( window_task );

    // add text and an exit button
    mPoint p( 20, h-60 );
    for( i=0; i<nlines; i++ )
    {
       (*wn) << *( new mWnCtlText( p, &fnt, txt[i] ) );
       p.y -= fnt.getcellh() + 1;
    }
    w /= 2;
    mRect r( w-30, 20, w+30, 40 );
    mWnCtlButton *but = new
          mWnCtlButton( r, mgr.getsysfont(), "Okay" );
    but->settask( button_task );
    (*wn) << (*but);

    // attach window to window manager
    mgr << (*wn);
}

//--------------------------------------------------------------------//

// PLATFORMS text

char *platform_txt[] =
{
   "*  Multiple OS platform support (some under development)...",
   "    DOS, Extended DOS, Windows, OS/2, Unix.",
   " ",
   "*  Support for many popular DOS graphics platforms...",
   "    BGI, Flash Graphics, Genus GX, MetaWINDOW, Microsoft.",
   " ",
   "*  Support for CIC's PenDOS and PENCENTRIC application",
   "    development via pen-based control objects.",
};

int platform_cnt = sizeof(platform_txt) / sizeof(char *);

// CAPABILITIES text

char *capabls_txt[] =
{
   "*  Window design tool for rapid prototyping and development.",
   " ",
   "*  A full range of general-purpose class objects plus many",
   "    pre-built dialogs (file selector, calculator, etc).",
   " ",
   "*  Fully capable CUA compliant window manager that handles",
   "    most window manipulation and user interaction.",
   " ",
   "*  EMS/XMS/Disk virtual arrays, BMP/PCX/GIF/TIF image",
   "    display, Super VGA support, HP Font support, and more.",
};

int capabls_cnt = sizeof(capabls_txt) / sizeof(char *);


//------------------------------------------------------------------//

mIconType1File *icnf = 0;

char icnmap[] =
{
   5, 0, 1, 4, 6, 7
};

char *ckblbls[] =
{
   "Printer Port",
   "Comm Port",
   "Spool File"
};

struct SpinData
{
   char  display_str[8];
   int   display_val;
   int   delta;
   int   vmin;
   int   vmax;
   SpinData( );
   SpinData( char *ds, int dv, int d, int mn, int mx );
  ~SpinData( );
};

SpinData::SpinData( )
{
   display_str[0] = 0;
   display_val = 0;
   delta = vmin = vmax = 0;
}

SpinData::SpinData( char *ds, int dv, int d, int mn, int mx )
{
   strcpy( display_str, ds );
   display_val = dv;
   delta = d;
   vmin = mn;
   vmax = mx;
}

SpinData::~SpinData( )
{
}

SpinData sd1( "1", 1, 1, 0, 10 );
SpinData sd2( "10", 10, 5, 0, 100 );
SpinData sd3( "0", 0, 1, -10, 10 );

char *control_types[] =
{
   "Aperture",
   "Button Group",
   "Button",
   "Check Box Group",
   "Check Box",
   "Combo Box",
   "Editable Field",
   "Gauges",
   "Icon",
   "List Box",
   "Bar/Box Menus",
   "Pen Input Field",
   "Pen Note Pad",
   "Pen Sketch Box",
   "Scroll Bars",
   "Spin Button",
   "Toggle Group",
   "Toggle",
   "Text",
};

int control_count = sizeof(control_types) / sizeof(char *);

char *desc_txt[] =
{
   "This window shows",
   "a few of the many",
   "controls supported",
   "by Menuet/CPP."
};

int desc_cnt = sizeof(desc_txt) / sizeof(char *);

int draw_icon_task( mMsg& msg, mWnControl& ctl )
{
   (*ctl.getwind()->getwnmgr()) << mMsg(mMOUSE,mHIDECURSOR);
   if( msg.verb == mENDDRAW )
   {
      mIconType1File *f = (mIconType1File *) ctl.getdata();
      mRect r = ctl.extent() >> 2;
      mPoint p( r.xmin, r.ymin );
      f->draw( p, ctl.tag );
   }
   (*ctl.getwind()->getwnmgr()) << mMsg(mMOUSE,mSHOWCURSOR);
   return 0;
}

char *sp_selector( int vect, void *data )
{
   SpinData *sd = (SpinData *) data;
   switch( vect )
   {
      case mNEXT :
           if( sd->display_val < sd->vmax )
              sd->display_val += sd->delta;
           sprintf( sd->display_str, "%d", sd->display_val );
           break;

      case mPREVIOUS :
           if( sd->display_val > sd->vmin )
              sd->display_val -= sd->delta;
           sprintf( sd->display_str, "%d", sd->display_val );
           break;
   }
   return sd->display_str;
}

mWindow * do_ctl_demo( mFont * fnt )
{
    if( icnf == 0 )
       icnf = new mIconType1File( "COMPUTER.ICN", arrXMS );

    mWindow *wn = new mWindow( "CONTROLS", 0, fnt,
                               450, 320, wBDRSIZABLE );
    wn->setstatus( yDESTROY, 1 );
    mRect r = wn->getclientrect();

    // some icons
    int iw = icnf->width() + 3;
    int ih = icnf->height() + 3;
    mRect ir( r.xmax-10-iw, r.ymax-10-ih,
              r.xmax-10,    r.ymax-10 );
    for( int i=0; i<6; i++ )
    {
       mWnCtlAperture *ap = new mWnCtlAperture( ir, RGNBOX );
       ir -= mPoint( 0, ih+1 );
       ap->setdata( icnf );
       ap->tag = icnmap[i];
       ap->settask( draw_icon_task );
       ap->setstatus( xPOSTDRAW, 1 );
       *wn << *ap;
    }

    // a check button group
    mPoint ip( r.xmin+10, r.ymax-110 );
    mWnCtlCheckGroup *cg = new
           mWnCtlCheckGroup( ip, fnt, "Hardcopy...", ckblbls, 1, 3, 0 );
    cg->setexclusive( 1 );
    cg->setpick( 1 );
    cg->setmustpick( 1 );
    *wn << *cg;

    // a set of spin buttons
    ip.set( r.xmin+10, r.ymax-141 );
    mWnCtlText *tx = new
       mWnCtlText( ip, fnt, "Parameters..." );
    *wn << *tx;
    ir.set( r.xmin+10, r.ymin+10, r.xmin+125, r.ymax-145 );
    mWnCtlAperture *ap = new mWnCtlAperture( ir, RGNBOX );
    *wn << *ap;
    ir.set( r.xmin+28, r.ymax-158, r.xmin+103, r.ymax-188 );
    ip.set( 0, 37 );
    mWnCtlSpinBut *sp = new
          mWnCtlSpinBut( ir, fnt, " ", sp_selector );
    sp->setdata( &sd1 );
    *wn << *sp;
    ir -= ip;
    sp = new mWnCtlSpinBut( ir, fnt, " ", sp_selector );
    sp->setdata( &sd2 );
    *wn << *sp;
    ir -= ip;
    sp = new mWnCtlSpinBut( ir, fnt, " ", sp_selector );
    sp->setdata( &sd3 );
    *wn << *sp;

    // a combo box
    ir.set( r.xmin+150, r.ymax-49, r.xmax-120, r.ymax-29 );
    mWnCtlComboBox *cb = new
          mWnCtlComboBox( ir, fnt, "Control Types...",
                          control_types, control_count );
    *wn << *cb;

    // some text
    ir.set( r.xmin+150, r.ymin+65, r.xmax-120, r.ymax-75 );
    *wn << *( new mWnCtlAperture( ir ) );
    ip.set( r.xmin+170, r.ymax-120 );
    for( i=0; i<desc_cnt; i++ )
    {
       *wn << *( new mWnCtlText( ip, fnt, desc_txt[i] ) );
       ip.y -= 20;
    }

    // an exit button
    ir.set( r.midx()-30, r.ymin+20, r.midx()+45, r.ymin+40 );
    mWnCtlButton *bu = new
       mWnCtlButton( ir, fnt, "Okay" );
    bu->settask( button_task );
    *wn << *bu;

    return wn;
}

//------------------------------------------------------------------//

// demo main menu definition

char *main_menu_labels[] =
{
   "~I~nfo",
   "~F~eatures",
   "~E~xamples",
   "~Q~uit"
};

mKey main_menu_keys[] =
{
   mKey( kbOTHER, kbALTI, kbALTKEY ),
   mKey( kbOTHER, kbALTF, kbALTKEY ),
   mKey( kbOTHER, kbALTE, kbALTKEY ),
   mKey( kbOTHER, kbALTQ, kbALTKEY ),
};

mMenuDef main_menu( main_menu_labels, main_menu_keys, 4, '~' );

// task for the main menu

int main_task( mMsg& msg, mWnControl& ctl )
{
   if( msg.verb == mSELECT )
   {
      switch( msg.param[0] )
      {
         case 1 : // INFO
         do_info( ctl.getwind()->getwnmgr() );
         return 1;

         case 4 : // QUIT
         do_quit( ctl );
         return 1;
      }
   }
   return 0;
}

//--------------------------------------------------------------------//

// features sub menu definition

char *feat_menu_labels[] =
{
   "~C~ontrols",
   "~G~eneral",
   "~P~latforms",
};

mKey feat_menu_keys[] =
{
   mKey( kbALPHANUM, 'C', 0 ),
   mKey( kbALPHANUM, 'G', 0 ),
   mKey( kbALPHANUM, 'P', 0 ),
};

mMenuDef feat_menu( feat_menu_labels, feat_menu_keys, 3, '~' );

// task for the features submenu

int feat_task( mMsg& msg, mWnControl& ctl )
{
   mWindowManager *wm = ctl.getwind()->getwnmgr();
   if( msg.verb == mSELECT )
   {
      switch( msg.param[0] ) // contains the submenu pick
      {
         case 1 :  *wm << *do_ctl_demo( fsys );
                   break;

         case 2 :  do_textpage( *wm, *fsml, "GENERAL FEATURES",
                                capabls_txt, capabls_cnt );
                   break;

         case 3 :  do_textpage( *wm, *fsml, "PLATFORMS",
                                platform_txt, platform_cnt );
                   break;
      }
   }
   return 0;
}

//--------------------------------------------------------------------//

// examples sub menu definition

char *exam_menu_labels[] =
{
   "~S~elect Palette",
   "~E~dit Palette",
   "~C~alculator",
   "~T~ime",
   "~L~oad Image",
   "~F~ree Memory",
   "~M~ouse Eyes",
};

mKey exam_menu_keys[] =
{
   mKey( kbALPHANUM, 'S', 0 ),
   mKey( kbALPHANUM, 'E', 0 ),
   mKey( kbALPHANUM, 'C', 0 ),
   mKey( kbALPHANUM, 'T', 0 ),
   mKey( kbALPHANUM, 'L', 0 ),
   mKey( kbALPHANUM, 'F', 0 ),
   mKey( kbALPHANUM, 'M', 0 ),
};

int nexam = sizeof(exam_menu_labels) / sizeof(char *);

mMenuDef exam_menu( exam_menu_labels, exam_menu_keys, nexam, '~' );

//--------------------------------------------------------------------//

// tasks for load image selection

int bmp_task( mMsg& msg, mWnControl& ctl )
{
   mImage *im = (mImage *) ctl.getdata();
   if( im == 0 ) return 0;
   mWindow *win = ctl.getwind();
   mRect r = win->getclientrect();
   mPoint p( r.xmin, r.ymax );
   if( msg.verb == mENDDRAW )
   {
      for( int i=0; i<im->imap->height(); i++ )
      {
         mGdMgr::drawbmap( p, im->imap->get(i), im->imap->width(), 1 );
         p.y--;
      }
   }
   else if( msg.verb == mDESTROY )
   {
      delete im;
      ctl.setdata( 0 );
   }
   return 0;
}

void loadimage( mWindowManager& wm, char *fn )
{
   if( fn[0] == 0 ) return;

   mImage *im = miLoadBMP( fn, arrXMS, &wm );

   if( (im == 0) || (im->status != imOKAY) )
   {
      wm << *notify( wm.getsysfont(), "ERROR",
                     "Error occurred attempting to load image." );
      beep();
   }
   else
   {
      int w = im->imap->width() + GetDefaultWnBdrThk() * 2;
      if( w > 600 ) w = 600;
      int h = im->imap->height() + GetDefaultWnBdrThk() * 2 + 20;
      if( h > 400 ) h = 400;
      mWindow *wn = new mWindow( fn, 0, wm.getsysfont(),
                                 w, h, wBDRSIZABLE );
      wn->setstatus( yDESTROY, 1 );
      mWnCtlAperture *ctl = new mWnCtlAperture( wn->getclientrect() );
      ctl->setstatus( xPOSTDRAW, 1 );
      ctl->setstatus( xVISIBLE, 0 );
      ctl->setdata( im );
      ctl->settask( bmp_task );
      *wn << *ctl;
      wm << *wn;
   }
}

// task for the examples submenu

int exam_task( mMsg& msg, mWnControl& ctl )
{
   if( msg.verb == mSELECT )
   {
      mWindowManager *wm = ctl.getwind()->getwnmgr();
      mFont *fnt = wm->getsysfont();
      switch( msg.param[0] ) // contains the submenu pick
      {
         case 1 :
              *wm << *create_palette_window(fnt);
              break;

         case 2 :
              *wm << *create_paledit_window(fnt);
              break;

         case 3 :
              *wm << *create_calc_window(fnt);
              break;

         case 4 :
              *wm << *create_clock_window(fnt);
              break;

         case 5 :
              *wm << *create_filebox_window(fnt,"Load BMP File",
                           filepath,filepick,loadimage);
              break;

         case 6 :
              *wm << *create_freemem_window(fnt);
              break;

         case 7 :
              *wm << *create_eyes_window(*wm);
              break;
      }
   }
   return 0;
}

//--------------------------------------------------------------------//

// setup menu structure and tasks

void do_setup( mWindow& wn )
{
    // get pointer to window's main menu
    mWnCtlBarMenu *mm = wn.getwnmenu();
    mm->settask( main_task );

    // create and hookup submenus
    mWnCtlBoxMenu *sm = new mWnCtlBoxMenu( wn.getwnfont(),
                                           &feat_menu );
    sm->settask( feat_task );
    mm->setpulldown( sm, 2 );

    sm = new mWnCtlBoxMenu( wn.getwnfont(),
                            &exam_menu );
    sm->settask( exam_task );
    mm->setpulldown( sm, 3 );

    // if in mono mode, remap some colors
    if( mGdMgr::getmaxcolors() == 2 )
    {
       mono_logo( );
       if( GetMonoStyle() >= 0 )
       {
          gold = 1;
          blue = 1;
       }
       else
       {
          gold = 0;
          blue = 0;
          flge->settxcolor( 1 );
          fsys->settxcolor( 1 );
          fsml->settxcolor( 1 );
          wn.getwnfont()->settxcolor( 1 );
       }
    }
}

//--------------------------------------------------------------------//

int main( int argc, char *argv[] )
{
    // set up some defaults
    if( getenv( "FG_DISPLAY" ) == 0 )
       putenv( "FG_DISPLAY=FG_VGA12" );
    SetWnRotation( 1 );
    SetDefaultPalette( pTomcat );
    SetMonoStyle( -1 );

    // instantiate the window manager
    mWindowManager *WM = new mWindowManager;

    // check what we're running on...
    if( mGdMgr::getdisprect().dely() < 300 )   // ...CGA
       SetDefaultWnBdrThk( 4 );
    else if( (mGdMgr::getdisprect().dely() < 400) &&
             (mGdMgr::getmaxcolors() == 16) )  // ...color EGA
       mGdMgr::setpalette( pSky );

    // load the auxiliary fonts
    fsys = new mHpFont( "SYS016.HP", 1, 255 );
    if( fsys->fstatus != fINIT )
    {
       delete fsys;
       fsys = &WM->systemfont();
    }
    fsml = new mHpFont( "NYK015.HP", 32, 127 );
    if( fsml->fstatus != fINIT )
    {
       delete fsml;
       fsml = &WM->systemfont();
    }
    flge = new mHpFont( "NYK020.HP", 32, 127 );
    if( flge->fstatus != fINIT )
    {
       delete flge;
       flge = &WM->systemfont();
    }
    flge->settxsp( 1 );

    // we'll use the FSYS font as our system font
    WM->setsysfont( fsys );

    // instantiate the application's main window
    mRect r = mGdMgr::getdisprect() >> 4;
    mWindow wn( "Menuet/CPP Demo",
                 &main_menu, &WM->systemfont(),
                 r.delx(), r.dely(), wBDRSIZABLE );

    // hookup submenus, etc
    do_setup( wn );

    // attach main window to the window manager
    *WM << wn;
    wn.setstatus( yBACKGROUND, 1 );

    // and display the info dialog
    load_logo();
    do_info( WM );

    // and run the application
    WM->run();

    // perform any clean up here
    if( icnf ) delete icnf;
    free_logo();

    // if created with the new operator, the window manager
    // must be expilicitly deleted to force destructor call
    delete WM;

    // following deletion of WM, we're in normal text mode
    printf( "......... M E N U E T / C P P ..........\n" );
    printf( "Autmumn  Hill  Software (r) Incorporated\n" );
    printf( "1145 Ithaca Drive, Boulder, CO 80303 USA\n" );
    printf( "Tel 303-494-8865 ...... Fax 303-494-7802\n" );

    return 0;
}
